
function get_html_translation_table (table, quote_style) {
    // Returns the internal translation table used by htmlspecialchars and htmlentities  
    // 
    // version: 1109.2015
    // discuss at: http://phpjs.org/functions/get_html_translation_table    // +   original by: Philip Peterson
    // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
    // +   bugfixed by: noname
    // +   bugfixed by: Alex
    // +   bugfixed by: Marco    // +   bugfixed by: madipta
    // +   improved by: KELAN
    // +   improved by: Brett Zamir (http://brett-zamir.me)
    // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
    // +      input by: Frank Forte    // +   bugfixed by: T.Wild
    // +      input by: Ratheous
    // %          note: It has been decided that we're not going to add global
    // %          note: dependencies to php.js, meaning the constants are not
    // %          note: real constants, but strings instead. Integers are also supported if someone    // %          note: chooses to create the constants themselves.
    // *     example 1: get_html_translation_table('HTML_SPECIALCHARS');
    // *     returns 1: {'"': '&quot;', '&': '&amp;'; '<': '&lt;', '>': '&gt;'}
    var entities = {},
    hash_map = {},        decimal;
    var constMappingTable = {},
    constMappingQuoteStyle = {};
    var useTable = {},
    useQuoteStyle = {}; 
    // Translate arguments
    constMappingTable[0] = 'HTML_SPECIALCHARS';
    constMappingTable[1] = 'HTML_ENTITIES';
    constMappingQuoteStyle[0] = 'ENT_NOQUOTES';    constMappingQuoteStyle[2] = 'ENT_COMPAT';
    constMappingQuoteStyle[3] = 'ENT_QUOTES';
    
    useTable = !isNaN(table) ? constMappingTable[table] : table ? table.toUpperCase() : 'HTML_SPECIALCHARS';
    useQuoteStyle = !isNaN(quote_style) ? constMappingQuoteStyle[quote_style] : quote_style ? quote_style.toUpperCase() : 'ENT_COMPAT'; 
    if (useTable !== 'HTML_SPECIALCHARS' && useTable !== 'HTML_ENTITIES') {
        throw new Error("Table: " + useTable + ' not supported');
        // return false;
    } 
    entities['38'] = '&amp;';
    if (useTable === 'HTML_ENTITIES') {
    	entities['39'] = '&apos;';
        entities['160'] = '&nbsp;';
        entities['161'] = '&iexcl;';        entities['162'] = '&cent;';
        entities['163'] = '&pound;';
        entities['164'] = '&curren;';
        entities['165'] = '&yen;';
        entities['166'] = '&brvbar;';        entities['167'] = '&sect;';
        entities['168'] = '&uml;';
        entities['169'] = '&copy;';
        entities['170'] = '&ordf;';
        entities['171'] = '&laquo;';        entities['172'] = '&not;';
        entities['173'] = '&shy;';
        entities['174'] = '&reg;';
        entities['175'] = '&macr;';
        entities['176'] = '&deg;';        entities['177'] = '&plusmn;';
        entities['178'] = '&sup2;';
        entities['179'] = '&sup3;';
        entities['180'] = '&acute;';
        entities['181'] = '&micro;';        entities['182'] = '&para;';
        entities['183'] = '&middot;';
        entities['184'] = '&cedil;';
        entities['185'] = '&sup1;';
        entities['186'] = '&ordm;';        entities['187'] = '&raquo;';
        entities['188'] = '&frac14;';
        entities['189'] = '&frac12;';
        entities['190'] = '&frac34;';
        entities['191'] = '&iquest;';        entities['192'] = '&Agrave;';
        entities['193'] = '&Aacute;';
        entities['194'] = '&Acirc;';
        entities['195'] = '&Atilde;';
        entities['196'] = '&Auml;';        entities['197'] = '&Aring;';
        entities['198'] = '&AElig;';
        entities['199'] = '&Ccedil;';
        entities['200'] = '&Egrave;';
        entities['201'] = '&Eacute;';        entities['202'] = '&Ecirc;';
        entities['203'] = '&Euml;';
        entities['204'] = '&Igrave;';
        entities['205'] = '&Iacute;';
        entities['206'] = '&Icirc;';        entities['207'] = '&Iuml;';
        entities['208'] = '&ETH;';
        entities['209'] = '&Ntilde;';
        entities['210'] = '&Ograve;';
        entities['211'] = '&Oacute;';        entities['212'] = '&Ocirc;';
        entities['213'] = '&Otilde;';
        entities['214'] = '&Ouml;';
        entities['215'] = '&times;';
        entities['216'] = '&Oslash;';        entities['217'] = '&Ugrave;';
        entities['218'] = '&Uacute;';
        entities['219'] = '&Ucirc;';
        entities['220'] = '&Uuml;';
        entities['221'] = '&Yacute;';        entities['222'] = '&THORN;';
        entities['223'] = '&szlig;';
        entities['224'] = '&agrave;';
        entities['225'] = '&aacute;';
        entities['226'] = '&acirc;';        entities['227'] = '&atilde;';
        entities['228'] = '&auml;';
        entities['229'] = '&aring;';
        entities['230'] = '&aelig;';
        entities['231'] = '&ccedil;';        entities['232'] = '&egrave;';
        entities['233'] = '&eacute;';
        entities['234'] = '&ecirc;';
        entities['235'] = '&euml;';
        entities['236'] = '&igrave;';        entities['237'] = '&iacute;';
        entities['238'] = '&icirc;';
        entities['239'] = '&iuml;';
        entities['240'] = '&eth;';
        entities['241'] = '&ntilde;';        entities['242'] = '&ograve;';
        entities['243'] = '&oacute;';
        entities['244'] = '&ocirc;';
        entities['245'] = '&otilde;';
        entities['246'] = '&ouml;';        entities['247'] = '&divide;';
        entities['248'] = '&oslash;';
        entities['249'] = '&ugrave;';
        entities['250'] = '&uacute;';
        entities['251'] = '&ucirc;';        entities['252'] = '&uuml;';
        entities['253'] = '&yacute;';
        entities['254'] = '&thorn;';
        entities['255'] = '&yuml;';
        // Others less common
    	entities['338'] = '&OElig;';
    	entities['339'] = '&oelig;';
        entities['352'] = '&Scaron;';
        entities['353'] = '&scaron;';
        entities['376'] = '&Yuml;';
        entities['402'] = '&fnof;';
        entities['710'] = '&circ;';
        entities['732'] = '&tilde;';
        entities['913'] = '&Alpha;';
        entities['914'] = '&Beta;';
        entities['915'] = '&Gamma;';
        entities['916'] = '&Delta;';
    	entities['917'] = '&Epsilon;';
    	entities['918'] = '&Zeta;';
    	entities['919'] = '&Eta;';
    	entities['920'] = '&Theta;';
    	entities['921'] = '&Iota;';
    	entities['922'] = '&Kappa;';
    	entities['923'] = '&Lambda;';
    	entities['924'] = '&Mu;';
    	entities['925'] = '&Nu;';
    	entities['926'] = '&Xi;';
    	entities['927'] = '&Omicron;';
    	entities['928'] = '&Pi;';
    	entities['929'] = '&Rho;';
    	entities['931'] = '&Sigma;';
    	entities['932'] = '&Tau;';
    	entities['933'] = '&Upsilon;';
    	entities['934'] = '&Phi;';
    	entities['935'] = '&Chi;';
    	entities['936'] = '&Psi;';
    	entities['937'] = '&Omega;';
    	entities['945'] = '&alpha;';
    	entities['946'] = '&beta;';
    	entities['947'] = '&gamma;';
    	entities['948'] = '&delta;';
    	entities['949'] = '&epsilon;';
    	entities['950'] = '&zeta;';	
    	entities['951'] = '&eta;';
    	entities['952'] = '&theta;';
    	entities['953'] = '&iota;';
    	entities['954'] = '&kappa;';
    	entities['955'] = '&lambda;';
    	entities['956'] = '&mu;';
    	entities['957'] = '&nu;';
    	entities['958'] = '&xi;';
    	entities['959'] = '&omicron;';
    	
    	entities['960'] = '&pi;';	
    	entities['961'] = '&rho;';
    	entities['962'] = '&sigmaf;';
    	entities['963'] = '&sigma;';
    	entities['964'] = '&tau;';
    	entities['965'] = '&upsilon;';
    	entities['966'] = '&phi;';
    	entities['967'] = '&chi;';
    	entities['968'] = '&psi;';
    	entities['969'] = '&omega;';
    	
    	entities['977'] = '&thetasym;';
    	entities['978'] = '&upsih;';
    	entities['982'] = '&piv;';
    	entities['8194'] = '&ensp;';
    	entities['8195'] = '&emsp;';
    	entities['8201'] = '&thinsp;';
    	entities['8204'] = '&zwnj;';
    	entities['8205'] = '&zwj;';
       	entities['8206'] = '&lrm;';
       	entities['8207'] = '&rlm;';

        entities['8211'] = '&ndash;';
        entities['8212'] = '&mdash;';
        entities['8216'] = '&lsquo;';
        entities['8217'] = '&rsquo;';
        entities['8220'] = '&ldquo;';
        entities['8221'] = '&rdquo;';
        entities['8226'] = '&bull;';
    } 
    if (useQuoteStyle !== 'ENT_NOQUOTES') {
        entities['34'] = '&quot;';
    }
    if (useQuoteStyle === 'ENT_QUOTES') {        entities['39'] = '&#39;';
    }
    entities['60'] = '&lt;';
    entities['62'] = '&gt;';
    
    // ascii decimals to real symbols
    for (decimal in entities) {
        if (entities.hasOwnProperty(decimal)) {
            hash_map[String.fromCharCode(decimal)] = entities[decimal];  
            // Also add reverse of it...
            hash_map[entities[decimal].substr(1)] = parseInt(decimal,10); 
        }
    }
    
    return hash_map;
}

function loadHashMap() 
{
    if(beersmith.hash_map === null)
        beersmith.hash_map = get_html_translation_table('HTML_ENTITIES', 'ENT_QUOTE');
}

// Decodes entities of the type &amp; as well as &#23;
function decodeEntities(str)
{
    loadHashMap();
    var hash_map = beersmith.hash_map;
    // Basically we look for all ampersands and try to find stuff
    var regex = /\&/g;
    var out = "";
    var sets = str.split(regex); // Splits it at the & points
    var i;
    if(sets.length === 0)
        return "";
    // console.log(sets);
    for(i=0; i<sets.length; i++)
    {
        var s = sets[i];
        if(s === "")
            continue;
        var ent=s.match(/\#[0-9]+;/); // Numeric matches
        if(ent !== null)
        {
            // console.log(ent[0]);
            // Look it up
            if(ent[0].length < 2)
                continue;
            var nent = ent[0].substring(1, ent[0].length-1);
            if(nent == "39" || nent == "34" || nent == "38")
            {
                sets[i] = s.replace(ent[0], "&#" + nent + ";");
                continue;
            }
            // console.log("nent:" +nent);
            sets[i] = s.replace(ent[0], String.fromCharCode(parseInt(nent,10)));
            // console.log("New str: " + sets[i]);
        }
        ent = s.match(/[a-zA-Z]+;/);
        if(ent !== null)
        {
           //  console.log(ent[0]);
            var aent = ent[0];
            // console.log(aent);
            if(hash_map.hasOwnProperty(aent))
            {
                var mychar = String.fromCharCode(hash_map[aent]);
                if(mychar == "&")
                	{
                	sets[i] = s.replace(ent[0], '&amp;');
                	}
                else
                	{
                	sets[i] = s.replace(ent[0], mychar);
                	}
            }
            else
                sets[i] = s.replace(ent[0], "&" + aent);
            // console.log("New str: " + sets[i]);
        }
            
    }
    var ret = sets.join("");
    // console.log(ret);
    return ret;
}